﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Runtime.InteropServices;
using System.Text;
using System.Threading.Tasks;

namespace ByteHandler
{
    [StructLayout(LayoutKind.Sequential, Size = 512, Pack = 1)]

    public struct MBR
    {
        [MarshalAs(UnmanagedType.ByValArray, SizeConst = 446)]
        public byte[] BootCode;

        [MarshalAs(UnmanagedType.ByValArray, SizeConst = 4)]
        public PartitionEntry [] Partitions;

        public ushort Signature;

        const ushort MBRSignature = 0xaa55;

        internal static bool TryParse(byte[] sector, out MBR mbr)
        {
            mbr = ByteArrayToStructure<MBR>(sector);
            return mbr.Signature == 0xaa55;
        }

        // which marshalling method is better? 
        // http://stackoverflow.com/questions/14465722/which-marshalling-method-is-better
        static T ByteArrayToStructure<T>(byte[] bytes) where T : struct
        {
            GCHandle handle = GCHandle.Alloc(bytes, GCHandleType.Pinned);
            T stuff = (T)Marshal.PtrToStructure(handle.AddrOfPinnedObject(), typeof(T));
            handle.Free();
            return stuff;
        }

        public override string ToString()
        {
            if (Signature != MBRSignature)
            {
                return "Unknown";
            }

            int count = 0;

            foreach (PartitionEntry partition in Partitions)
            {
                if (partition.SizeOfSector != 0)
                {
                    count++;
                }
            }

            return string.Format("MBR, # of Partitions: {0}", count);
        }
    }
}
