﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Runtime.InteropServices;
using System.Text;
using System.Threading.Tasks;

namespace ByteHandler
{
    public class MasterBootRecord
    {
        private const int MAX_PATH = 260;

        public void Read()
        {
            LowLevelDiskAccess lld = new LowLevelDiskAccess();

            List<string> driveList = lld.GetDriveList();
            if (driveList.Count == 0)
            {
                return; // Is it possible?
            }

            for (int i = 0; i < driveList.Count; i++)
            {
                int bytesPerSector = lld.BytesPerSector(i);
                string driveName = driveList[i];
                Console.WriteLine(string.Format("{0}: bytesPerSector == {1}", driveName, bytesPerSector));

                byte[] sector = lld.DumpSector(driveName, 0, bytesPerSector);

                MBR mbr;
                bool result = MBR.TryParse(sector, out mbr);

                if (result == true)
                {
                    foreach (var part in mbr.Partitions)
                    {
                        Console.WriteLine(part);
                    }
                }

                Console.WriteLine("");
            }
        }

        // Enumerate physical drives in Windows
        // https://www.virag.si/2010/02/enumerate-physical-drives-in-windows/
        private static List<string> BuildDeviceMap()
        {
            string[] logicalDrives = Environment.GetLogicalDrives();
            List<string> localDeviceMap = new List<string>(logicalDrives.Length);
            StringBuilder lpTargetPath = new StringBuilder(MAX_PATH);

            foreach (string drive in logicalDrives)
            {
                string lpDeviceName = drive.Substring(0, 2);

                int result = UnsafeNativeMethods.QueryDosDevice(lpDeviceName, lpTargetPath, MAX_PATH);
                if (result == 0)
                {
                    if (Marshal.GetLastWin32Error() == 5)
                    {
                        localDeviceMap.Clear();
                        throw new UnauthorizedAccessException();
                    }
                }

                localDeviceMap.Add(lpTargetPath.ToString());
            }

            return localDeviceMap;
        }
    }
}
