﻿
using System;
using System.Runtime.InteropServices;

namespace ByteHandler
{
    [StructLayout(LayoutKind.Explicit, Size = 16, Pack = 1)]
    public unsafe struct PartitionEntry
    {
        [FieldOffset(0)]
        public byte BootFlag;

        [FieldOffset(1)]
        fixed byte _startingCHSAddress[3];

        public unsafe int StartingCHSAddress
        {
            get
            {
                fixed (byte* pBytes = _startingCHSAddress)
                {
                    return ToInt32(pBytes);
                }
            }
        }

        [FieldOffset(4)]
        public byte Type;

        [FieldOffset(5)]
        fixed byte _endingCHSAddress[3];

        public unsafe int EndingCHSAddress
        {
            get
            {
                fixed (byte *pBytes = _endingCHSAddress)
                {
                    return ToInt32(pBytes);
                }
            }
        }

        [FieldOffset(8)]
        public uint StartingLBAAddress;

        [FieldOffset(12)]
        public uint SizeOfSector;

        unsafe int ToInt32(byte* pBytes)
        {
            byte[] intBytes = new byte[4];

            intBytes[0] = *pBytes;
            intBytes[1] = *(pBytes + 1);
            intBytes[2] = *(pBytes + 2);

            return BitConverter.ToInt32(intBytes, 0);
        }

        public override string ToString()
        {
            if (SizeOfSector == 0)
            {
                return "(undefined)";
            }

            return string.Format("Bootable = {0,5}, Type = {1,-8}, SectorSize = 0x{2,8:x8}, CHS = 0x{3,6:x6} ~ 0x{4,6:x6}, BeginLBA = 0x{5,8:x8}",
                BootFlag == 0x80, GetPartitionTypeName(Type), SizeOfSector, StartingCHSAddress, EndingCHSAddress, StartingLBAAddress);
        }

        // Partition type
        // https://en.wikipedia.org/wiki/Partition_type
        private string GetPartitionTypeName(byte type)
        {
            switch (type)
            {
                case 0x07:
                    return "NTFS";

                case 0x0F:
                    return "Extended";

                case 0xee:
                    return "GPT(protective)";

                default:
                    return string.Format("Uknown(0x{0:x})", type);
            }
        }
    }
}